==> 2.0.0-beta10&file=out_yasapi-nt-2.0.0-beta10-src.tar.gz: out_yasapi-nt-2.0.0-beta10/gyasapi_property.c <==
/*
 * yasapi_property.h
 *
 * Yet Another (WA)SAPI (YASAPI) New Technology (NT) Output (out_yasapi-nt)
 * Copyright (C) 2018  Peter Belkner (pbelkner@snafu.de)
 *
 * out_yasapi-nt is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * out_yasapi-nt is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with out_yasapi-nt.  If not, see <https://www.gnu.org/licenses/>.
 */
#include <yasapi.h>

///////////////////////////////////////////////////////////////////////////////
#define SIZEOF(T,member) \
    (sizeof ((T *)NULL)->member)
#define PROPERTY(lpVtbl,T,member) \
    { lpVtbl,L ## #member,offsetof(T,member),SIZEOF(T,member) }
#define PROPERTY_INT(T,member) \
    PROPERTY(&gcPropertyVtblInt,T,member)
#define PROPERTY_DBL(T,member) \
    PROPERTY(&gcPropertyVtblDbl,T,member)
#define PROPERTY_STR(T,member) \
    PROPERTY(&gcPropertyVtblStr,T,member)

///////////////////////////////////////////////////////////////////////////////
static void PropertyIntRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  int *pnData=(int *)(data+pProperty->offs);
  const int *pnDefault=(const int *)(ref+pProperty->offs);

  if (path[0]) {
    *pnData=GetPrivateProfileIntW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      *pnDefault,             // _In_ INT     nDefault,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
  else
    *pnData=*pnDefault;

  DVWRITELNW(L"\"%s\" \"%s\": %d (%d)",lpAppName,pProperty->lpKeyName,
      *pnData,*pnDefault);
}

static void PropertyIntWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  const int *pnData=(const int *)(data+pProperty->offs);
  const int *pnRef=(const int *)(ref+pProperty->offs);
  wchar_t buf[32];

  if (path[0]&&*pnData!=*pnRef) {
    _itow(*pnData,buf,10);

    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblInt={
  .read=PropertyIntRead,
  .write=PropertyIntWrite,
};

///////////////////////////////////////////////////////////////////////////////
static void PropertyDblRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  enum { eBufSize=128 };
  double *pxData=(double *)(data+pProperty->offs);
  const double *pxDefault=(const double *)(ref+pProperty->offs);
  wchar_t buf[eBufSize];

  if (path[0]) {
    swprintf(buf,eBufSize,L"%lf",*pxDefault);

    GetPrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_  LPCTSTR lpDefault,
      buf,                    // _Out_ LPTSTR  lpReturnedString,
      eBufSize,               // _In_  DWORD   nSize,
      path                    // _In_ LPCTSTR lpFileName
    );

    swscanf(buf,L"%lf",pxData);
  }
  else
    *pxData=*pxDefault;

  DVWRITELNW(L"\"%s\" \"%s\": %f (%f)",lpAppName,pProperty->lpKeyName,*pxData,
      *pxDefault);
}

static void PropertyDblWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  enum { eBufSize=128 };
  const double *pxData=(const double *)(data+pProperty->offs);
  const double *pxRef=(const double *)(ref+pProperty->offs);
  wchar_t buf[eBufSize];

  if (path[0]&&*pxData!=*pxRef) {
    swprintf(buf,eBufSize,L"%lf",*pxData);

    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblDbl={
  .read=PropertyDblRead,
  .write=PropertyDblWrite,
};

///////////////////////////////////////////////////////////////////////////////
static void PropertyStrRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  wchar_t *pData=(wchar_t *)(data+pProperty->offs);
  const wchar_t *pDefault=(const wchar_t *)(ref+pProperty->offs);

  if (path[0]) {
    GetPrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      pDefault,               // _In_  LPCTSTR lpDefault,
      pData,                  // _Out_ LPTSTR  lpReturnedString,
      pProperty->dwSize,      // _In_  DWORD   nSize,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
  else
    wcscpy(pData,pDefault);

  DVWRITELNW(L"\"%s\" \"%s\": \"%s\" (\"%s\")",lpAppName,pProperty->lpKeyName,
      pData,pDefault);
}

static void PropertyStrWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  const wchar_t *pData=(const wchar_t *)data;
  const wchar_t *pRef=(const wchar_t *)ref;

  if (path[0]&&wcscmp(pData,pRef)) {
    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      pData,                  // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblStr={
  .read=PropertyStrRead,
  .write=PropertyStrWrite,
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesDialog[]={
  PROPERTY_INT(OptionsDialog,nPosX),
  PROPERTY_INT(OptionsDialog,nPosY),
  PROPERTY_INT(OptionsDialog,nPage),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesGtk[]={
  PROPERTY_STR(OptionsGtk,path),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesCommon[]={
  PROPERTY_STR(OptionsCommon,id),
  PROPERTY_INT(OptionsCommon,bLatency),
  PROPERTY_INT(OptionsCommon,eMono2Stereo),
  PROPERTY_INT(OptionsCommon,bVolume),
  PROPERTY_INT(OptionsCommon,bDisconnect),
  PROPERTY_INT(OptionsCommon,bSurround),
  PROPERTY_INT(OptionsCommon,bGapless),
  PROPERTY_INT(OptionsCommon,pos.eConfig),
  PROPERTY_INT(OptionsCommon,pos.eAbout),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesDevice[]={
  PROPERTY_INT(OptionsDevice,eMode),
  PROPERTY_INT(OptionsDevice,bAutoConvertPCM),
  PROPERTY_INT(OptionsDevice,bSRCDefaultQuality),
  PROPERTY_INT(OptionsDevice,eStrategy),
  PROPERTY_INT(OptionsDevice,bTimerPeriodic),
  PROPERTY_INT(OptionsDevice,bAlignmentDance),
#if defined (YASAPI_AV_THREAD_CHARACTERISTICS) // [
  PROPERTY_INT(OptionsDevice,bProAudio),
#endif // ]
  PROPERTY_INT(OptionsDevice,ePeriod),
  PROPERTY_INT(OptionsDevice,eBitdepth),
  PROPERTY_DBL(OptionsDevice,xRingSize),
  PROPERTY_DBL(OptionsDevice,xMaxLatencyAssumed),
  PROPERTY_DBL(OptionsDevice,xPlaybackBegin),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
void PropertiesRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, void *data, const void *ref)
{
  while (pProperty->lpVtbl) {
    pProperty->lpVtbl->read(pProperty,path,lpAppName,data,ref);
    ++pProperty;
  }
}

void PropertiesWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const void *data, const void *ref)
{
  while (pProperty->lpVtbl) {
    pProperty->lpVtbl->write(pProperty,path,lpAppName,data,ref);
    ++pProperty;
  }
}

==> 2.3.0-beta10&file=out_yasapi-nt-2.3.0-beta10-src.tar.gz: out_yasapi-nt-2.3.0-beta10/gyasapi_property.c <==
/*
 * yasapi_property.h
 *
 * Yet Another (WA)SAPI (YASAPI) New Technology (NT) Output (out_yasapi-nt)
 * Copyright (C) 2015-2019  Peter Belkner (pbelkner@snafu.de)
 *
 * out_yasapi-nt is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * out_yasapi-nt is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with out_yasapi-nt.  If not, see <https://www.gnu.org/licenses/>.
 */
#include <yasapi.h>

///////////////////////////////////////////////////////////////////////////////
#define SIZEOF(T,member) \
    (sizeof ((T *)NULL)->member)
#define PROPERTY(lpVtbl,T,member) \
    { lpVtbl,L ## #member,offsetof(T,member),SIZEOF(T,member) }
#define PROPERTY_INT(T,member) \
    PROPERTY(&gcPropertyVtblInt,T,member)
#define PROPERTY_DBL(T,member) \
    PROPERTY(&gcPropertyVtblDbl,T,member)
#define PROPERTY_STR(T,member) \
    PROPERTY(&gcPropertyVtblStr,T,member)

///////////////////////////////////////////////////////////////////////////////
static void PropertyIntRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  int *pnData=(int *)(data+pProperty->offs);
  const int *pnDefault=(const int *)(ref+pProperty->offs);

  if (path[0]) {
    *pnData=GetPrivateProfileIntW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      *pnDefault,             // _In_ INT     nDefault,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
  else
    *pnData=*pnDefault;
}

static void PropertyIntWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  const int *pnData=(const int *)(data+pProperty->offs);
  const int *pnRef=(const int *)(ref+pProperty->offs);
  wchar_t buf[32];

  if (path[0]&&*pnData!=*pnRef) {
    _itow(*pnData,buf,10);

    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblInt={
  .read=PropertyIntRead,
  .write=PropertyIntWrite,
};

///////////////////////////////////////////////////////////////////////////////
static void PropertyDblRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  enum { eBufSize=128 };
  double *pxData=(double *)(data+pProperty->offs);
  const double *pxDefault=(const double *)(ref+pProperty->offs);
  wchar_t buf[eBufSize];

  if (path[0]) {
    swprintf(buf,eBufSize,L"%lf",*pxDefault);

    GetPrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_  LPCTSTR lpDefault,
      buf,                    // _Out_ LPTSTR  lpReturnedString,
      eBufSize,               // _In_  DWORD   nSize,
      path                    // _In_ LPCTSTR lpFileName
    );

    swscanf(buf,L"%lf",pxData);
  }
  else
    *pxData=*pxDefault;
}

static void PropertyDblWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  enum { eBufSize=128 };
  const double *pxData=(const double *)(data+pProperty->offs);
  const double *pxRef=(const double *)(ref+pProperty->offs);
  wchar_t buf[eBufSize];

  if (path[0]&&*pxData!=*pxRef) {
    swprintf(buf,eBufSize,L"%lf",*pxData);

    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      buf,                    // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblDbl={
  .read=PropertyDblRead,
  .write=PropertyDblWrite,
};

///////////////////////////////////////////////////////////////////////////////
static void PropertyStrRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, char *data, const char *ref)
{
  wchar_t *pData=(wchar_t *)(data+pProperty->offs);
  const wchar_t *pDefault=(const wchar_t *)(ref+pProperty->offs);

  if (path[0]) {
    GetPrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      pDefault,               // _In_  LPCTSTR lpDefault,
      pData,                  // _Out_ LPTSTR  lpReturnedString,
      pProperty->dwSize,      // _In_  DWORD   nSize,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
  else
    wcscpy(pData,pDefault);
}

static void PropertyStrWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const char *data, const char *ref)
{
  const wchar_t *pData=(const wchar_t *)data;
  const wchar_t *pRef=(const wchar_t *)ref;

  if (path[0]&&wcscmp(pData,pRef)) {
    WritePrivateProfileStringW(
      lpAppName,              // _In_ LPCTSTR lpAppName,
      pProperty->lpKeyName,   // _In_ LPCTSTR lpKeyName,
      pData,                  // _In_ LPCTSTR lpString,
      path                    // _In_ LPCTSTR lpFileName
    );
  }
}

static IPropertyVtbl gcPropertyVtblStr={
  .read=PropertyStrRead,
  .write=PropertyStrWrite,
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesAbout[]={
  PROPERTY_INT(OptionsAbout,nPosX),
  PROPERTY_INT(OptionsAbout,nPosY),
  PROPERTY_INT(OptionsAbout,nPage),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesConfig[]={
  PROPERTY_INT(OptionsConfig,nPosX),
  PROPERTY_INT(OptionsConfig,nPosY),
  PROPERTY_INT(OptionsConfig,nPage),
  PROPERTY_INT(OptionsConfig,common.nPage),
  PROPERTY_INT(OptionsConfig,monitor.nPage),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesGtk[]={
  PROPERTY_STR(OptionsGtk,path),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesCommon[]={
  PROPERTY_STR(OptionsCommon,id),
  PROPERTY_INT(OptionsCommon,bLatency),
  PROPERTY_INT(OptionsCommon,eMono2Stereo),
  PROPERTY_INT(OptionsCommon,eVolume),
  PROPERTY_INT(OptionsCommon,eDither),
  PROPERTY_INT(OptionsCommon,bDisconnect),
  PROPERTY_INT(OptionsCommon,bSurround),
  PROPERTY_INT(OptionsCommon,bGapless),
  PROPERTY_INT(OptionsCommon,eThemeEngineTail),
  PROPERTY_INT(OptionsCommon,eThemePadding),
  PROPERTY_INT(OptionsCommon,eThemeStart),
  PROPERTY_INT(OptionsCommon,eThemeWritten),
  PROPERTY_INT(OptionsCommon,present.bEnabled),
  PROPERTY_DBL(OptionsCommon,present.xPeriod),
  PROPERTY_INT(OptionsCommon,bBoost),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
const Property gcaPropertiesDevice[]={
  PROPERTY_INT(OptionsDevice,eMode),
  PROPERTY_INT(OptionsDevice,bAutoConvertPCM),
  PROPERTY_INT(OptionsDevice,bSRCDefaultQuality),
  PROPERTY_INT(OptionsDevice,eStrategy),
  PROPERTY_INT(OptionsDevice,bTimerPeriodic),
  PROPERTY_INT(OptionsDevice,bAlignmentDance),
  PROPERTY_INT(OptionsDevice,bProAudio),
  PROPERTY_INT(OptionsDevice,ePeriod),
  PROPERTY_INT(OptionsDevice,eBitdepth),
  PROPERTY_DBL(OptionsDevice,xMaxLatencyAssumed),
  PROPERTY_DBL(OptionsDevice,xPlaybackBegin),
  { NULL,NULL,-1,0 }
};

///////////////////////////////////////////////////////////////////////////////
void PropertiesRead(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, void *data, const void *ref)
{
  while (pProperty->lpVtbl) {
    pProperty->lpVtbl->read(pProperty,path,lpAppName,data,ref);
    ++pProperty;
  }
}

void PropertiesWrite(const Property *pProperty, const wchar_t *path,
    const wchar_t *lpAppName, const void *data, const void *ref)
{
  while (pProperty->lpVtbl) {
    pProperty->lpVtbl->write(pProperty,path,lpAppName,data,ref);
    ++pProperty;
  }
}
